#!/usr/bin/python

from common.utils import init_logging, do_lp_login, get_basename

logger = init_logging("ul10nStatsCalc")

class LanguageSet(object):
    """A container class for a set of languages of type Language object"""

    def __init__(self):
        """Initialize the languages dictionary as a container of Language
        objects.
        """
        self.languages = {}

    def add_language(self, language):
        """Add a Language object to the languages dictionary"""
        self.languages[language.code] = language

    def _get_languages_len(self):
        """Return how many languages this set has"""
        return len(self.languages)

    length = property(_get_languages_len)

    def get_all_languages(self, options, config=None):
        """Fetch all available languages either known to Launchpad or listed
        in a configuration file.
        """
        logger.info('Getting language list...')

        self.config = config

        if self.config is None:
            # Get data from Launchpad if no configuration file was specified
            max_tries = 2
            tries = 0

            launchpad = do_lp_login()

            try:
                lp_languages = launchpad.languages
            except urllib2.URLError, e:
                tries += 1
                if tries >= max_tries or (e.code != 502 and e.code != 504):
                    logger.warning("Failed (%d): %s" % (e.code, url))
                    raise

                logger.warning("Retrying (%d): %s" % (e.code, url))
                time.sleep(3)

            # Populate the language dictionary with the visible languages
            # known to Launchpad
            for lp_language in lp_languages:
                language = Language(lp_language.code,
                                    lp_language.english_name,
                                    lp_language.visible)
                self.add_language(language)
        else:
            logger.info('Using language config file...')
            # A config file was specified: read the list of languages from
            # there. Format: 2 initial comments and subsequently
            # '|'-separated values: 'code | english_name | visible', where
            # 'visible' is either 't' or 'f' (True or False, respectively)
            # if custom option, omit third value: 't' or 'f' and its '|'
            started = 2
            for line in open(config):
                if started > 0:
                    started -= 1
                elif '|' in line:
                    items = line.split('|')
                    code = items[0]
                    english_name = items[1]
                    if len(items) == 3:
                        visible = items[2]
                    else:
                        visible = 't'
                    #if options.custom and options.languages_file:
                    #    code, english_name = line.split('|')
                    #    visible = "t"
                    #else:
                    #    code, english_name, visible = line.split('|')
                    language = Language(code.strip(),
                                        english_name.strip(),
                                        visible.strip() == "t")
                    if language.visible:
                        # Populate the language dictionary with the visible
                        # languages specified
                        self.add_language(language)

        logger.debug('Obtained {0!s} languages'.format(self.length))

    def __getitem__(self, code):
        """If languages is a LanguageSet, it allows us to use the
        languages[code] shortcut to access the LanguageSet dictionary of
        languages (instead of languages.languages[code].
        """
        return self.languages[code]

    def __iter__(self):
        """If languages is a LanguageSet, in iteration operations
        (e.g. for, while, etc.) it allows us to use the languages shortcut to
        iterate over the languages dictionary instead of having to specify the
        dictionary explicitly. E.g. 'for i in languages' instead of
        'for i in languages.languages'.
        """
        return self.languages.itervalues()

    def __contains__(self, code):
        """If languages is a LanguageSet, in container operations
        (e.g. in, etc.) it allows us to use the languages shortcut to
        find out whether a language is in a set, instead of having to specify
        dictionary explicitly. E.g. 'if i in languages' instead of
        'if i in languages.languages'.
        """
        return code in self.languages


class Language(object):
    """A language class. Its attributes are a subset of the language class
    from the Launchpad API.
    """
    def __init__(self, code, english_name, visible):
        """Initialize the ISO 639-2 code, the readable English name and
        whether the language is visible in Launchpad.
        """
        self.code = code
        self.english_name = english_name
        self.visible = visible

    def __str__(self):
        """Returns the English name string for any operations that deal with
        the object's string representation, e.g. 'print language'.
        """
        return self.english_name


