#!/usr/bin/env python

# utils - Provides several helper functions
#
# Author: Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 20100 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import logging
import gzip
from utils import init_logging
import logging.config


class Templates():
    """A class to represent and provide access to LP templates"""

    def __init__(self, template_file, settings_path):
        self.templates_all = self._get_templates_from_file(template_file, 
                                settings_path,)
        self.templates_enabled = self._make_enabled_templates(
                            self.templates_all)
        self.template_src = self._make_template_src_dict(self.templates_enabled)
        self.src_domains = self._make_src_domains_dict(self.templates_enabled)
        self.domain_src = self._make_domain_src_dict(self.templates_enabled)
        self.templates_project = []

# A set representing current working set of templates. Each set item is a
# tuple: src pkgs, template (the lp template name)
        self.num_messages_in_working_templates = 0
        self.working_src_template_set = set()
        self.templates_priority = []
# The number of translatable messages in the current working set of
# templates:
        self.num_messages_in_working_templates
        self.templates_priority_blacklist = []
        self.templates_conf_file = []
        self.templates_conf_file_blacklist = []
        self.src_pkgs_not_in_lang_packs = []
        return

    def add_src_pkgs_not_in_lang_packs(self, srcs):
        self.src_pkgs_not_in_lang_packs = srcs
        return

    def blacklist_by_conf_file(self, templates, options, settings):

        if len(options.blacklist) != 0:
            f_blacklist = options.blacklist
        else:
            f_blacklist = 'config/blacklist.' + settings['distro_codename'] + \
'.conf'

        # create or get blacklist
        conf_file_whitelist = list()
        conf_file_blacklist = list()
        blacklist = []

        # Set up blacklist from config/blacklist.conf
        for line in open(f_blacklist):
            src, domain= line.split('|')
            domain = domain.strip()
            #print "domain: " + domain
            blacklist.append(domain)

        for record in templates.templates_project:
            if record['domain'] in blacklist:
                conf_file_blacklist.append(record)
            else:
                conf_file_whitelist.append(record)
        self.templates_conf_file = conf_file_whitelist
        self.templates_project = conf_file_whitelist
        self.templates_conf_file_blacklist = conf_file_blacklist

        return conf_file_whitelist, conf_file_blacklist

    def blacklist_by_priority(self, templates, settings):

        # create or get blacklist
        priority_whitelist = list()
        priority_blacklist = list()

        # Set up blacklist from priorities
        cutoff_priority = settings['blacklist_priority']

        for record in templates.templates_project:
            if record['priority'] <= cutoff_priority:
                priority_blacklist.append(record)
            else:
                priority_whitelist.append(record)

        self.templates_priority = priority_whitelist
        self.templates_project = priority_whitelist
        self.templates_priority_blacklist = priority_blacklist

        return priority_whitelist, priority_blacklist


    def _get_templates_from_file(self, f_templates, settings):
        """Returns list of dictionaries for all LP templates"""
 
        logger = init_logging("ul10nTemplates")
        if not os.path.exists(f_templates):
            logger.error("Path to templates file does not exist. Stopping")
            sys.exit(1)
        lines = gzip.open(f_templates, 'rb').readlines()
        start = 0
        templates = []
        for line in lines:
            if start < 2:
                start += 1
                continue
            if line.startswith("("): break

            template_dict = {}
            items = line.split("|")
            template_dict['src'] = items[0].strip()
            template_dict['domain'] = items[1].strip()
            template_dict['template'] = items[2].strip()
            template_dict['total'] = items[3].strip()
            template_dict['enabled'] = items[4].strip()
            template_dict['langpack'] = items[5].strip()
            template_dict['priority'] = items[6].strip()
            template_dict['last_updated'] = items[7].strip()
            templates.append(template_dict)
        return templates

    def _make_template_src_dict(self, templates_enabled):
        template_src = dict()
        for record in templates_enabled:
            template_src[record['template']] = record['src']
        return template_src

    def _make_domain_src_dict(self, templates_enabled):
        domain_src = dict()
        for record in templates_enabled:
            domain_src[record['domain']] = record['src']
        return domain_src

    def _make_src_domains_dict(self, templates_enabled):
        src_domains = dict()
        for record in templates_enabled:
            if record['src'] not in src_domains.keys():
                src_domains[record['src']] = [record['domain']]
            else:
                if record['domain'] not in src_domains[record['src']]:
                     src_domains[record['src']].append(record['domain'])
        return src_domains

    def _make_enabled_templates(self, templates_all):
        enabled = []
        for record in templates_all:
            if record['enabled'] == "t":
                enabled.append(record)
        return enabled

    def get_all_templates(self):
        return self.templates_all

    def get_enabled_templates(self):
        return self.templates_enabled

    def get_project_templates_from_src_pkgs(self, src_pkgs):
        templates_project = []
        for record in self.templates_enabled:
            if record['template'] in src_pkgs:
                templates_project.append(record)
        self.templates_project = templates_project
        self._get_working_src_template_set()
        return templates_project

    def get_total_number_messages_in_working_templates(self):
        for record in self.templates_project:
            self.num_messages_in_working_templates += int(record['total'])
        return self.num_messages_in_working_templates

    def _get_working_src_template_set(self):
        for record in self.templates_project:
            self.working_src_template_set.add((record['src'], record['template']))
        return self.working_src_template_set

    def get_all_blacklisted_templates():
        blacklist = []

        for record in self.templates_conf_file_blacklist:
            if record['domain'] not in blacklist['domain']:
                if record['domain'] in \
self.templates_priority_blacklist['domain']:
                    blacklist.append(record)

        self.blacklist = blacklist
        return blacklist

    def get_templates_from_binary_pkgs(self, pkgs):
        pass
        return

    def get_templates_from_domains(self, domains):
        pass
        return

