#!/usr/bin/env python

# ul10n_stats_calc - Calculates translation statistics for a given set of
# source packages and domains, and produces a report
#
# Author: Danilo Segan <danilo@canonical.com>, with modifications by
#   David Planella <david.planella@ubuntu.com>
#   Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2009, 2010, 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import Languages
import sys
import codecs
import os
import urllib2
import datetime
import gzip
import subprocess
from common.utils import init_logging, get_basename
from common.templates import html_report_template
sys.path.append(os.path.join(os.getcwd(), 'lib/python2.6/site-packages'))
from gviz_api import gviz_api

logger = init_logging("ul10nStatsCalc")

def addToLanguage(stats, language, translated, total):

    if language in stats:
        stats[language]['translated'] += translated
        stats[language]['total'] += total
    else:
        stats[language] = {
            'translated': translated,
            'total': total,
            }

def calculate_stats(options, settings, templates, lp_srcpkgs):

    # To hold all statistics.
    stats = {}

    #file containing LP data exports:
    pkg_stats = '{0}/{1}-package-stats.log.gz'.format(options.data_dir, settings['distro_codename'])

    # set up some vars
    translated_percent_min = int(settings['translated_percent_min'])

    # get the total number of translatable messages in the curret set of
    # working templates
    TOTAL = int(templates.get_total_number_messages_in_working_templates())
    #print "TOTAL %s" % TOTAL

    # get the working templates as a set of tuples (src, pot)
    working_templates = templates.working_src_template_set

    # get up languages
    languages = Languages.LanguageSet()
    languages.get_all_languages(options, options.languages_file)

    # Skip 2 lines from the package stats file
    started = 2

    # go through lp_srcpkgs object. For each, if the srcpkg/template
    # is in working templates, add the translation data to stats
    for record in lp_srcpkgs.srcpkgs:

        if (record['src'], record['template']) in working_templates \
            and record['language'] in languages:
            addToLanguage(stats,
                    record['language'],
                    int(record['translated']),
                    int(record['total']),
                    )
                ## Prints the per-language detailed template statistics
                ## Caveat: if there are no translations for a template in a
                ## given language, the template will not be listed

    # Get a list of all languages for which there is a translation
    languages_available = stats.keys()

    # Sort the list of available languages
    languages_available.sort(cmp=lambda a, b:
                        cmp(stats[b]['translated'], stats[a]['translated']))

    # This list will contain the language statistics data in the format
    # the gviz_api.DataTable expects
    language_stats = []
    languages_supported = 0
    for lang in languages_available:
        if lang in languages:
            #print "TOTAL %d" % TOTAL
            print str(stats[lang]['translated'])
            translated_percent = 100.00 * int(stats[lang]['translated']) / TOTAL
            language_stats.append({
                "language": str(languages[lang]),
                "translated": translated_percent
                })
            if round(translated_percent) >= translated_percent_min:
                languages_supported += 1
#                logger.debug('{0}. Supported language: {1}'.format(
#                                                        languages_supported,
#                                                        languages[lang]))

    languages_total = len(languages_available)

    description = {
        "language": ("string", "Language"),
        "translated": ("number", "Translated")
        }

    # Load the data into a gviz_api.DataTable
    data_table = gviz_api.DataTable(description)
    data_table.LoadData(language_stats)

    data_json = data_table.ToJSon(
                columns_order=("language", "translated"),
                order_by=[("translated", "desc"), ("language", "asc")]
                )

    if options.custom:
        description = {
            "src_pkg": ("string", "Source Package"),
            'template': ('string', "Template"),
            'domain': ('string', "Gettext Domain")
                    }
        data = gviz_api.DataTable(description)
        included_data_table, included_json = \
                get_custom_html_data(options, settings,
                templates.templates_project, data)
        blacklisted_priority_data, blacklisted_priority_json = \
                get_custom_html_data(options, settings,
                templates.templates_priority_blacklist, data)
        blacklisted_file_data, blacklisted_file_json = \
                get_custom_html_data(options, settings, 
                templates.templates_conf_file_blacklist, data)

        description = {
            "src": ("string", "Source Package"),
                    }
        data_ex = gviz_api.DataTable(description)

        excluded_data_table, excluded_json = \
                get_custom_html_data_one_col(options, settings,
                templates.src_pkgs_not_in_lang_packs, data_ex, "src")

    basename = get_basename(options,settings)
    report_filename = os.path.join(options.output_dir, basename + '.html')

    # Log the date of creation for the report
    gen_date = str(datetime.datetime.utcnow())
    variables = locals()
    # Add necessary variables for the report
    variables['gen_date'] = gen_date
    variables['distro_release'] = settings['distro_release']
    variables['project'] = options.custom_name
    variables['basename'] = basename
    f = codecs.open(report_filename, 'w', 'utf-8')
    if options.custom:
        from common.templates import custom_report as template
        from common.templates import custom_src_excluded as excluded
        f_ = codecs.open(os.path.join(options.output_dir, basename + '-excluded.html'), 'w', 'utf-8')
        f_.write(excluded.ubuntu_template % variables)
        f_.close()
        from common.templates import custom_src_included as included
        f_ = codecs.open(os.path.join(options.output_dir, basename + '-included.html'), 'w', 'utf-8')
        f_.write(included.ubuntu_template % variables)
        f_.close()
        from common.templates import custom_blacklisted_file as blacklisted_file
        f_ = codecs.open(os.path.join(options.output_dir, basename + '-blacklisted-file.html'), 'w', 'utf-8')
        f_.write(blacklisted_file.ubuntu_template % variables)
        f_.close()
        from common.templates import custom_blacklisted_priority as blacklisted_priority
        f_ = codecs.open(os.path.join(options.output_dir, basename + '-blacklisted-priority.html'), 'w', 'utf-8')
        f_.write(blacklisted_priority.ubuntu_template % variables)
        f_.close()
        #print "variables: " + str(variables)
    else:
        from common.templates import html_report_template as template
    f.write(template.ubuntu_template % variables)
    f.close()

    logger.info('Report generated at {0}'.format(report_filename))





def get_custom_html_data_one_col(options, settings, records, data, col):
    #basename = get_basename(options, settings, col)

    templates_working = list()
    templates_working_dict = dict()
    idx = -1
    for record in records:
        idx += 1
        if record in templates_working_dict.keys(): continue
        else:
            templates_working.append({
                    col:record,
                    })
            templates_working_dict[record] = True
    data.LoadData(templates_working)
    json = data.ToJSon(
                    #columns_order=(col)
                        )
    return data, json

def get_custom_html_data(options, settings, final_templates, data):
    #basename = get_basename(options, settings)

    templates_working = list()
    templates_working_dict = dict()
    for record in final_templates:
        if record['domain'] in templates_working_dict.keys(): continue
        else:
            templates_working.append({
                    'src_pkg':record['src'],
                    'template':record['template'],
                    'domain' :record['domain']
                    })
            templates_working_dict[record['domain']] = True
    data.LoadData(templates_working)
    json = data.ToJSon(
                    columns_order=("src_pkg", "template", 'domain'),
                    order_by=[('src_pkg', 'asc'), ('template', 'asc'),
('domain', 'asc')]
                        )
    return data, json
