#!/usr/bin/env python

# utils - Provides several helper functions
#
# Author: Kyle Nitzsche <kyle.nitzsche@canonical.com>
#
# Copyright (c) 20100 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import glob
import subprocess
import re
import tempfile
import atexit
import shutil
from ConfigParser import RawConfigParser
import logging
import logging.config

def get_settings():
    SETTINGS_FILE = os.path.join(os.getcwd(), "config", "settings.conf")
    if not os.path.exists(SETTINGS_FILE):
        print "Error. Can't open {0}. Stopping.".format(SETTINGS_FILE)
        sys.exit(1)

    config = RawConfigParser()
    config.read(SETTINGS_FILE)

    settings = dict()

    # Read all configuration items in the settings file. This allows
    # us dynamically adding new items and getting them read automatically
    config_items = config.items('defaults')
    config_items.extend(config.items('distros'))

    for config_item in config_items:
        settings[config_item[0]] = config_item[1]
    
    # Infer the distro release from its codename in the settings
    # and add it explicitly to the settings dictionary
    settings['distro_release'] = config.get('distros', settings['distro_codename'])

    return settings

def get_basename(options, settings):
    if options.custom:
        if options.custom_name is None:
            basename = 'custom-{0}-{1}'.format(settings['distro_id'],settings['distro_release'])
        else:
            basename = '{0}'.format(options.custom_name)
    else:
        basename = '{0}-{1}'.format(settings['distro_id'],settings['distro_release'])

    return basename

def get_working_pot_templates(settings):
    templates = os.path.join(os.getcwd(), 'data', settings['distro_codename'] + "-templates.txt")
    if not os.path.exists(templates):
        print >>sys.stderr, "Error: " + template + " file does not exist. Stopping."
        sys.exit(1)
    return templates

def init_logging(logger_name):
    settings_path = os.path.join(os.getcwd(), "config", "settings.conf")
    logging.config.fileConfig(settings_path)

    # Create the logger
    logger = logging.getLogger(logger_name)
    
    return logger

def do_lp_login():

    from launchpadlib.launchpad import Launchpad

    cachedir = tempfile.mkdtemp()
    atexit.register(shutil.rmtree, cachedir)

    logger.info("Logging in to Launchpad... ")

    # TODO: error checking, especially if we're working offline
    launchpad = Launchpad.login_anonymously('Ubuntu Translation stats',
            'production', cachedir)

    return launchpad

def get_ooo_pot_mapping(path):

    lines = open(path, 'r').readlines()

    ooo_pot_mapping = dict()
    started = 1
    for line in lines:
        if started > 0:
            started -= 1
            continue

        if '|' in line:
            (pot, path, rename_pot) = line.split('|')
            pot = pot.strip()
            path = path.strip()
            rename_pot = rename_pot.strip()
            ooo_pot_mapping[path] = rename_pot.strip()
    
    return ooo_pot_mapping

def get_ooo_raw_data(path, ooo_pot_mapping):
    
    for currentFile in glob.glob( os.path.join(path, '*') ):
        cmd = 'LC_ALL=C msgfmt --statistics -o /dev/null {0}'.format(currentFile)
        if os.path.isdir(currentFile):
            #print 'Got a directory: ' + currentFile
            get_ooo_raw_data(currentFile, ooo_pot_mapping)
        else:
            basename, extension = os.path.splitext(os.path.basename(currentFile))
            if extension == '.pot':
                #print >>sys.stderr, "Processing template: " + currentFile
                result = subprocess.Popen(cmd, shell=True, cwd="./", stderr=subprocess.PIPE ).communicate()
                total = re.findall(r'[0-9]+', result[1].split(',')[1])[0]
                sourcepackage = 'openoffice.org'
                path = re.findall(r'debian/l10n/po/.*', currentFile)[0]
                template_name = ooo_pot_mapping[path]
                translation_domain = template_name
                enabled = 't'
                languagepack = 'f'
                # We set the priority of the UI and doc translations according to
                # https://wiki.ubuntu.com/Translations/TemplatesPriority
                if not 'helpcontent2' in currentFile.split('/'):
                    priority = 6500
                else:
                    priority = 3000
                # We could read the date from the POT file, but we don't need it, thus we fake it
                date_last_updated = datetime.datetime.utcnow().__str__()
                #print ' {0:<31}| {1:<31}| {2:<31}| {3:<31}| {4:<31}| {5:<31}| {6:<31}| {7:<31}'.format(sourcepackage, translation_domain, template_name, total, enabled, languagepack, priority, date_last_updated)  
            elif extension == '.po':
                #print >>sys.stderr, "Processing file: " + currentFile
                result = subprocess.Popen(cmd, shell=True, cwd="./", stderr=subprocess.PIPE ).communicate()
                status = result[1].split(',')
                statusDict = {}
                statusStrings = ['translated', 'fuzzy', 'untranslated']
                
                # Read the number of translated, fuzzy and untranslated strings and put them in a dictionary
                for item in status:
                    for statusStr in statusStrings:
                        statusRegexp = r'[0-9]+ {0}'.format(statusStr)
                        if (re.findall(statusRegexp, item)):
                            statusDict[statusStr] = re.findall(r'[0-9]+', item)[0]

                # msgfmt gives us only status if the count is non-zero. Fill the missing statuses.
                for statusStr in statusStrings:
                    if not statusStr in statusDict:
                        statusDict[statusStr] = 0

                sourcepackage = 'openoffice.org'
                pot = os.path.join(os.path.dirname(currentFile), (os.path.split(os.path.dirname(currentFile))[1] + '.pot'))
                path = re.findall(r'debian/l10n/po/.*', pot)[0]
                template_name = ooo_pot_mapping[path]
                translation_domain = template_name
                # We could read the date from the POT file, but we don't need it, thus we fake it.
                # Same with the PO file
                date_pot = datetime.datetime.utcnow().__str__()
                date_translation = date_pot
                lang = basename.replace('-', '_')
                translated = statusDict['translated']
                unreviewed = statusDict['fuzzy']
                untranslated = statusDict['untranslated']
                total = int(translated) + int(unreviewed) + int(untranslated)
                changed = 0
                print ' {0:<31}| {1:<31}| {2:<31}| {3:<31}| {4:<31}| {5:<31}| {6:<31}| {7:<31}| {8:<31}| {9:<31}'.format(sourcepackage, translation_domain, template_name, date_pot, date_translation, lang, translated, unreviewed, changed, total)  

logger = init_logging("Utils")

if __name__ == "__main__":
    # For Maverick OO.o translations were fetched from:
    # https://launchpadlibrarian.net/56015550/openoffice.org_3.2.1-6ubuntu2_i386_translations.tar.gz
    ooo_pot_mapping = get_ooo_pot_mapping('../data/oo-o-pots-mapping.txt')
    get_ooo_raw_data('../data/source/debian/l10n/po', ooo_pot_mapping)
    sys.exit(0)
