#!/usr/bin/env python

# This script calculates statistics of the percentage of translation per
# language for all languages in Ubuntu by calling all involved functions in the
# right order.
#
# Author: David Planella <david.planella@ubuntu.com> and Kyle Nitzsche
# <kyle.nitzsche@canonical.com>
#
# Copyright (c) 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import urllib
import os
import subprocess
from optparse import OptionParser
from common import utils
from common import LpTemplates
from common import LpSrcPkgs
from common import ul10n_stats_calc
from common import DistroSeeds

CONFIG_DIR = 'config'
EXTRA_SRC_PKGS = [
    'debian-installer',
    'ubiquity',
    'ubiquity-slideshow-ubuntu',
    'gfxboot-theme-ubuntu',
    ]

logger = utils.init_logging("ul10nStats")

def options_parse():
    """Parses the command line and config file options"""

    settings = utils.get_settings()

    USAGE = "%prog [OPTIONS]"

    DESCRIPTION = """Lists all translation templates and source packages in \
the LiveCD for the given distro version."""

    parser = OptionParser(usage = USAGE, description = DESCRIPTION)
    parser.add_option("-s",
                    "--stats-url",
                    dest="stats_url",
                    default="http://people.canonical.com/~danilo/ubuntu",
                    help="URL to the directory containing the LP data export \
gz files.")
    parser.add_option("-x",
                    "--do-not-download-stats",
                    dest="do_not_download_stats",
                    action="store_true",
                    default=False,
                    help="Use existing LP stats files -- do not download \
them.")
    parser.add_option("-p",
                    "--disable-priority-blacklist",
                    dest="disable_priority_blacklist",
                    action="store_true",
                    default=False,
                    help="Disables using priorities to generate blacklist.")
    parser.add_option("-d",
                    "--data-dir",
                    dest="data_dir",
                    default="data",
                    help="Directory that contains the LP exported data files.")
    parser.add_option("-b",
                    "--template-blacklist",
                    dest="blacklist",
                    default='',
                    help="Specify the file containing the templates to \
blacklist during the calculation of statistics")
    parser.add_option("-l",
                    "--languages-file",
                    dest="languages_file",
                    default=None,
                    help="File with languages for report.")
    parser.add_option("-c",
                    "--custom",
                    dest="custom",
                    action="store_true",
                    default=False,
                    help="Turn on custom (non-Ubuntu) mode")
    parser.add_option("-n",
                    "--custom-name",
                    dest="custom_name",
                    default=None,
                    help="Name of custom project. Requires -c argument.")
    parser.add_option("-m",
                    "--manifest-file",
                    dest="manifest_file",
                    default=None,
                    help="Manifest file to read bin pkgs from and on which \
stats are based. Requires -c argument.")
    parser.add_option("-f",
                    "--disable-file-blacklist",
                    dest="disable_file_blacklist",
                    action="store_true",
                    default=False,
                    help="Disables using conf file blacklist.")
    parser.add_option("-u",
                    "--ubuntu",
                    dest="ubuntu",
                    action="store_true",
                    default=False,
                    help="Use Ubuntu as scope for statistics.")
    parser.add_option("-r",
                    "--ubuntu-release",
                    dest="ubuntu_release",
                    default=None,
                    help="Ubuntu release code name, for example: maverick")
    parser.add_option("-o",
                    "--output-dir",
                    dest="output_dir",
                    default="output",
                    help="Directory in which to place gnerated output.")

    (options, args) = parser.parse_args()

# reset distro codename from command line arg, if used
    if options.ubuntu_release is not None:
        settings['distro_codename']=options.ubuntu_release

# and try to make the output dir, stop if cannot
    if not os.path.exists(options.output_dir):
        cmd = ['mkdir', '-p', options.output_dir]
        try:
            subprocess.call(cmd)
        except:
            print >>sys.stderr, "Error: Cannot create output dir: " + options.output_dir \
+ ". Stopping"
            sys.exit(1)

# copy html assets to output dir:
    cmd = ['cp', '-r', 'common/css', options.output_dir]
    subprocess.call(cmd)
    cmd = ['cp', '-r', 'common/img', options.output_dir]
    subprocess.call(cmd)


    return options, settings

def get_raw_data(distro_codename, options):
    logger.info("Fetching stats data...")
    for stats_file in ('potemplate', 'package'):
        url = '{0}/{1}-{2}-stats.log.gz'.format(
            options.stats_url, 
            distro_codename, 
            stats_file,
            )
        path = os.path.join(options.data_dir, os.path.basename(url))
        urllib.urlretrieve(url, path)

def get_src_pkgs_from_binary_pkgs(binaries):
    not_found = list()
    src_pkgs = list()

    for items in binaries:
        cmd = ['apt-cache', 'showsrc', items[0]]
        try:
            res = subprocess.Popen(
            cmd, 
            cwd="./", 
            stdout=subprocess.PIPE,
            ).communicate()
        except:
            print >>sys.stderr, "Error: subprocess exception. stopping."
            sys.exit(1)
            continue
        src_pkg = res[0][res[0].find(" ") + 1:res[0].find("\n")]
        if len(src_pkg) == 0:
            not_found.append(items[0])
            continue
        if src_pkg not in src_pkgs:
            src_pkgs.append(src_pkg)

        f_ = open('pkgs_not_found', 'w')
        for pkg in not_found:
            f_.write(pkg + '\n')
        f_.flush()
        f_.close()

    src_pkgs.sort()

    return src_pkgs, not_found

def get_bin_pkgs_from_manifest(path):
    if not os.path.exists(path):
        print >> sys.stderr, "Error: " + path + "does not exist. Stoppping."
        sys.exit(1)
    f = open(path, 'r')
    lines = f.readlines()
    f.close()

    binaries = list(tuple())
    for line in lines:
        items = line.split()
        binaries.append([items[0],items[1]])

    return binaries

def write_template(the_list, name, options, settings):
    #print "the list: \n"  + str(the_list)
    basename = utils.get_basename(options,settings)
    f_name = basename + name
    f = open(os.path.join(os.getcwd(), "reports", f_name), 'w')
    #the_list_ = list(the_set)
    the_list.sort()
    for dictionary in the_list:
        idx = -1
        for item in dictionary.values():
            if idx != -1:
                f.write(" | ")
            idx += 1
            f.write(item)
        f.write('\n')
    f.flush()
    f.close()

    return

def log(templates, options, settings):
    # write blaclisted files
    write_template(
        templates.templates_conf_file_blacklist, 
        "-blacklist-file", 
        options, 
        settings,
        )
    write_template(
        templates.templates_priority_blacklist, 
        "-blacklist-priority", 
        options, 
        settings,
        )
    #log_blacklist(blacklisted, "all", options, settings)

    write_template(templates.templates_enabled, "-superset", options, settings)
    write_template(templates.templates_project, "-working", options, settings)
#TODO: restore/complete this
    #if options.manifest_file:
        #write_template(
        #src_pkgs_not_found, 
        #"src-pkgs-not-found", 
        #options, 
        #settings,
        #)

    return

def get_distro_src_pkgs(settings):
    distro_seeds = DistroSeeds.DistroSeeds(
                        settings['distro_id'],
                        settings['distro_codename']
                        )
    distro_seeds.get_sourcepackages()
    return distro_seeds.sourcepackages

def add_distro_extras(templates):
    templates_extra = templates
    print str(len(templates))
#TODO: this is redundant with var set at top of file. fix that
#AND: the list should be configurable and optionally read from file.
    EXTRA_SRC_PKGS = [
        'debian-installer',
        'ubiquity',
        'ubiquity-slideshow-ubuntu',
        'gfxboot-theme-ubuntu',
        ]
    valid = list()
    for record in templates:
        print record['src']
        if record['src'] in EXTRA_SRC_PKGS:
            templates_extra.append(record)

    return templates_extra

def get_custom_src_pkgs(options, settings, templates, src_pkgs):
    src_pkgs_not_found = list()
    domains = list()
    for src_pkg in src_pkgs:
        if src_pkg in templates.src_domains.keys():
            for domain in templates.src_domains[src_pkg]:
                domains.append(domain)
        else:
            src_pkgs_not_found.append(src_pkg)
    superset = list()
    invalid = list()
    for domain in domains:
        if domain in templates.domain_src.keys():
            for record in templates.templates_enabled:
                if record['domain'] == domain:
                     superset.append(record['src'])
    #print "== len(superset): " + str(len(superset))
    return superset, src_pkgs_not_found


def main():

    # Get the command line options and settings from the config file
    options, settings = options_parse()

    # Fetch the raw data export files to be used as the source for the
    # calculation, unless overidden by command arg
    if not options.do_not_download_stats:
        get_raw_data(settings['distro_codename'], options)

    # create templates object
    templates = LpTemplates.Templates(
            "data/{0}-potemplate-stats.log.gz".format(
                settings['distro_codename'],
                ),
            settings,
            )
    # get all templates
    templates_all = templates.get_all_templates()
    print "Templates, all: " + str(len(templates_all))

    # get enabled LP templates
    templates_enabled = templates.get_enabled_templates()
    print "Templates, enabled: " + str(len(templates_enabled))

    # get project source pkgs
    if options.ubuntu:
        project_src_pkgs = get_distro_src_pkgs(settings)
        print "Project src pkgs: " + str(len(project_src_pkgs))
        # add those source pkgs that are in distro (live-cd) but not in seeds
        for src in EXTRA_SRC_PKGS: project_src_pkgs.add(src)
        print "Project src pkgs with EXTRAS: " + str(len(project_src_pkgs))
    if options.custom:
        if not options.manifest_file:
            print >> sys.stderr, "Error: manifest file not provided with \
command line arg but required. Stopping."
            sys.exit(1)
        binary_pkgs = get_bin_pkgs_from_manifest(options.manifest_file)
        src_pkgs, not_found = get_src_pkgs_from_binary_pkgs(binary_pkgs)
        project_src_pkgs, not_found = get_custom_src_pkgs(
                            options, 
                            settings, 
                            templates, 
                            src_pkgs
                            )
        print "Project src pkgs: " + str(len(project_src_pkgs))
        print "Project src pkgs, NOT FOUND: " + str(len(not_found))
        templates.add_src_pkgs_not_in_lang_packs(not_found)
        print "Templates, not found: " + \
str(len(templates.src_pkgs_not_in_lang_packs))

    # get project_templates by filter with project src pkgs
    templates_project = templates.get_project_templates_from_src_pkgs(
                                    project_src_pkgs
                                    )
    print "Templates, project: " + str(len(templates_project))

    if not options.disable_file_blacklist:
        # get templates after conf file blacklisting
        templates_conf_file, templates_conf_file_blacklist = \
                    templates.blacklist_by_conf_file(
                                        templates,
                                        options,
                                        settings,
                                        )
        print "Templates, after conf_file blacklisting: " + \
str(len(templates_conf_file))
        print "Templates, blacklisted conf_file: " + \
            str(len(templates_conf_file_blacklist))

    if not options.disable_priority_blacklist:
        # get templates after priority blacklisting
        templates_priority, templates_priority_blacklist = \
                    templates.blacklist_by_priority(
                                        templates,
                                        settings,
                                        )
        print "Templates, after priority blacklisting: " + \
str(len(templates_priority))
        print "Templates, blacklisted priority: " + \
            str(len(templates_priority_blacklist))

# TODO: remove this testing code
    src_pkgs_file = '{0}/{1}-package-stats.log.gz'.format(options.data_dir, \
settings['distro_codename'])

        #print "src_pkgs_file: %s" % src_pkgs_file
    lp_srcpkgs = LpSrcPkgs.LpSrcPkgs(src_pkgs_file)

        #print lp_srcpkgs[5]['domain']
        #print str(lp_srcpkgs.srcpkgs[5])

        #sys.exit(1)

    # Calculate the unified, per language translation statistics from the
    # working templates set and output them in a report
    ul10n_stats_calc.calculate_stats(options, settings, templates, lp_srcpkgs)

    # write log files
    log(templates, options, settings)

if __name__ == "__main__":
    main()
    sys.exit(0)
